#!/usr/bin/env python
#-*- coding: UTF-8 -*-

import sys
if sys.version < '3':
    import urllib2
else:
    import urllib.request, urllib.error, urllib.parse
import zipfile
import wx
from distutils.file_util import copy_file
from distutils.dir_util import copy_tree
from . import mumapopup
from   .mumamisc import *

class InstallDialog(wx.Dialog):
    ID_START = wx.NewId()
        
    def __init__(self, parent, config, logger, url):
        """  """
        wx.Dialog.__init__(self, parent, -1, TITLE + ' - install new release', style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        self.logger = logger
        self.parent = parent
        self.config = config
        self.tologger = config['Misc']['actionlog'] == 'True'
        btnadj = int(config['Misc']['norm'] == 'False')
        self.url = url
        self.logline = 0

        sizer = wx.BoxSizer(wx.VERTICAL)
        
        sb = wx.StaticBox(self, -1, ' Installation status messages ')
        row1 = wx.StaticBoxSizer(sb, wx.VERTICAL)       
        self.tc = wx.TextCtrl(self, -1, style = wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_DONTWRAP)      
        self.tc.Enable(False)
        background_color = wx.SystemSettings_GetColour(READONLY_COLOR)
        self.tc.SetBackgroundColour(background_color)
        row1.Add(self.tc, 2, wx.EXPAND | wx.ALL, GAP)
        
        sb = wx.StaticBox(self, -1, ' Installation progress ')
        row2 = wx.StaticBoxSizer(sb, wx.VERTICAL)
        self.gauge = wx.Gauge(self, -1)
        row2.Add(self.gauge, 0, wx.EXPAND | wx.ALL, GAP)

        self.butstart = wx.Button(self, self.ID_START, '&Start installation')
        self.butclose = wx.Button(self, wx.ID_CLOSE)
        self.Bind(wx.EVT_BUTTON, self.onButtonStart, id = self.ID_START)
        self.Bind(wx.EVT_BUTTON, self.onButtonClose, id = wx.ID_CLOSE)     
        row3 = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: row3.AddStretchSpacer()
        row3.Add(self.butstart, btnadj, wx.GROW | wx.RIGHT, GAP)
        row3.Add(self.butclose, btnadj, wx.GROW | wx.LEFT, GAP)

        sizer.Add(row1, 3, wx.GROW | LRT, BORDER)
        
        sizer.Add(row2, 0, wx.GROW | LRT, BORDER)
        sizer.Add(row3, 0, wx.GROW | wx.ALL, BORDER) # + GAP)
        self.SetSizer(sizer)
        self.SetMinSize(wx.Size(440, 400))
        self.Fit()
        self.Centre()
        self.logText('the new release to be installed is %s' % url[-17:])
        self.logText("press 'Start installation'")
    #-------------------------------------------------------------------------------------------

    def logText(self, text, color = None):
        if color != None:
            start = self.tc.GetInsertionPoint()
        self.tc.WriteText('- %s\n' % text)
        if self.tologger:
            self.logger('Installer - %s' % text)
        if color != None:
            stop = self.tc.GetLastPosition()
            self.tc.SetStyle(start, stop, wx.TextAttr(color))
        self.tc.ShowPosition(-1)
        self.logline += 1
        if self.logline % 4 == 0: wx.SafeYield(self)
    #-------------------------------------------------------------------------------------------

    def backupCurrentFiles(self, bkupdir):
        dirlist = os.listdir(APPPATH)
        self.gauge.SetRange(len(dirlist))
        for index, name in enumerate(dirlist):
            spname = os.path.join(APPPATH, name)
            if os.path.isfile(spname) and not os.path.islink(spname) and not name.endswith('.zip'):
                copy_file(spname, bkupdir)
            elif os.path.isdir(spname):
                if not name in ['backup', '.bzr']:
                    copy_tree(spname, os.path.join(bkupdir, name))
            self.gauge.SetValue(index + 1)
            if index % 4 == 0: wx.SafeYield(self)
        wx.SafeYield(self)
    #-------------------------------------------------------------------------------------------

    def installNewFiles(self, zipfname):
        zfile = zipfile.ZipFile(zipfname, 'r')
        names = zfile.namelist()
        self.gauge.SetRange(len(names))
        for index, name in enumerate(names):
            if name.endswith('/') or name == 'READ.ME': continue
            outfname = os.path.join(APPPATH, name[9:])
            cf = zfile.read(name)
            self.logText(outfname)
            outfile = open(outfname, 'wb')
            outfile.write(cf)
            outfile.close()
            self.gauge.SetValue(index + 1)
        wx.SafeYield(self)
    #-------------------------------------------------------------------------------------------
         
    def onButtonStart(self, event):
        self.butstart.Enable(False)
        self.logText('making backup of current .ini file ...')       
        inifileupdate = file(self.config['Version']['inifilename'].replace('.ini', '.ini.update'), 'w')
        self.config.write(inifileupdate)
        inifileupdate.close()

        try:
            self.logText('making backup of current installation ...')
            bkupdir = os.path.join(APPPATH, 'backup')
            if not os.path.exists(bkupdir):
                os.mkdir(bkupdir, 0777)
            if not os.path.exists(bkupdir):
                raise
            self.logText('copying files ...') 
            self.backupCurrentFiles(bkupdir)
            self.logText('backup completed')
        except:
            self.logText('could not backup current installation - giving up', RED)
            return
        
        self.gauge.SetValue(0)
        try:
            self.logText('trying to connect to download site ...')
            if sys.version < '3':
                mysock = urllib2.urlopen(self.url, timeout = 5)
            else:
                mysock = urllib.request.urlopen(self.url, timeout = 5)
        except:
            self.logText('could not connect to download site - giving up', RED)
            return
        try:
            self.logText('starting download - this can take a while ...') 
            file2save = mysock.read()
            self.logText('download finished')
        except:
            self.logText('could not download file - giving up', RED)
            return       
        localfname = os.path.join(APPPATH, self.url[-17:])
        try:   
            self.logText('trying to store downloaded file locally ...')
            localfile = open(localfname, 'wb')
            localfile.write(file2save)
            localfile.close()
            self.logText("downloaded file stored locally as '%s'" % localfname)
        except:
            self.logText('could not store file - giving up', RED)
            return
        self.logText('installing new files ...')
        self.butclose.Enable(False)
        wx.SafeYield(self)
        try:
            self.installNewFiles(localfname)
            self.logText('new release successfully installed')
        except:            
            self.logText('could not install files - giving up', RED)
            self.butclose.Enable(True)
            return
        self.tc.WriteText('\n')
        self.logText('shutdown and restart %s to finish the installation' % TITLE)
        
        info = 'To complete the update process\nshut down and restart ' + TITLE
        updatewin = mumapopup.UpdatePopup(self, info)
        pos = self.parent.ClientToScreen((0,0))  # pos is the left top edge of the win
        sz = self.parent.GetSize()               # sz is the size of the win
        updatewin.Position((pos[0] + sz[0] / 2 - 150, pos[1] - 50 - sz[1] / 2), (0, sz[1]))
        updatewin.Popup()
        self.butclose.Enable(True)        
    #-------------------------------------------------------------------------------------------

    def onButtonClose(self, event):
        self.Destroy()

# - class InstallDialog(wx.Dialog):--------------------------------------------------------------
 