#!/usr/bin/env python
#-*- coding: UTF-8 -*-
 
import sys
import wx
import os
import os.path
import re
from   time import strftime

LINUX = sys.platform.startswith('linux')
if LINUX:
    import pwd
    USER = pwd.getpwuid(os.getuid())[0]
    WILDEXEC = 'All files (*)|*'
else:
    import getpass
    USER = getpass.getuser()
    WILDEXEC = 'Program file (*.exe)|*.exe|' \
               'All files (*)|*' 
TITLE = 'MusicMan'
VERNUM = '6.36-3'
RELEASEDATE = '2016-02-07'
IMAGEDIR = TITLE.lower() + 'images'
APPICON = TITLE.lower() + '.ico'
HOMEDIR = os.path.expanduser("~")
APPPATH = os.path.dirname(sys.argv[0])
ICONPATH = os.path.join(APPPATH, IMAGEDIR)
VERSION = '%s - Version %s (%s)' % (TITLE, VERNUM, USER)
URL_BASE = 'http://www.die-mersmanns.de/www/'
URL_DOWNLOAD = URL_BASE + 'images/%s/download' % (TITLE.lower())
URL_WEBSITE = URL_BASE + 'index.php/projekt-%s/downloads' % (TITLE.lower())
SEPSPACE = os.sep + ' '

RED = wx.Colour(255, 0, 0)                       # Error, no db connection
GREEN = wx.Colour(0, 255, 0)                     # ready
BLUE = wx.Colour(0, 90, 240)                     # scan dir to db
YELLOW = wx.Colour(255, 255, 0)                  # query is executing
LBLUE = wx.Colour(66, 199, 243)                  # task is running
LRED = wx.Colour(253, 190, 185)                  # displimit reached
GREY = wx.Colour(127, 127, 127)
MAGENTA = wx.Colour(255, 0, 255)
BLACK = wx.Colour(0, 0, 0)
READONLY_COLOR = wx.SYS_COLOUR_INACTIVECAPTION
if "wxMSW" in wx.PlatformInfo:
    SCALE = 16
    PSTRING = 'wxMSW'
elif "wxGTK" in wx.PlatformInfo:
    SCALE = 22
    PSTRING = 'wxGTK'
else:
    SCALE = 48
    PSTRING = ''
PYVERSION = 'running on Python - Version %s' % sys.version.split()[0]
WXVERSION = 'using wxPython - Version %s' % wx.VERSION_STRING
PLATFORMINFO = 'wx Platform Info - %s' % PSTRING

BORDER = 10
GAP = 5
LRT = wx.LEFT | wx.RIGHT | wx.TOP

WILDCSV = 'csv file (*.csv)|*.csv|' \
          'All files (*.*)|*.*'
WILDM3U = 'm3u file (*.m3u)|*.m3u|' \
          'All files (*.*)|*.*'

ACTION_LOG_SIZE =  64 * 1024
BACKUP_LOG_SIZE = 512 * 1024

FFMPEG2MP3 = '-map_metadata 0:s:0 -ab 256k'
FFMPEG2OGG = '-map_metadata 0:s:0 -acodec libvorbis -aq 8'

MYTOP25 = 'My Top 25'
#-------------------------------------------------------------------------------------------

def isSpecialId(id_title):
    return id_title.endswith('00') or id_title.startswith('99')
#-------------------------------------------------------------------------------------------

def getTagInfo(config, id_title = None, mfilepath = None):
    ismp3 = False
    meta = {}
    bitrate = 0
    duration = '00:00:00'

    if config['Misc']['mutagenpresent'] == 'False':
        return meta, ismp3, bitrate, duration
    else:
        from   mutagen.mp3 import MP3
        from   mutagen.oggvorbis import OggVorbis
        from   mutagen.easyid3 import EasyID3
        import mutagen.id3

    if mfilepath == None:
        found, mfilepath = findTitleExt(config, id_title)
    else:
        found = True
    if found:
        if mfilepath.endswith('mp3') or mfilepath.endswith('MP3'):
            meta = MP3(mfilepath, ID3 = EasyID3)
            try:
                bitrate = meta.info.bitrate
                duration = toString(int(meta.info.length + 0.5))
                meta.add_tags(ID3 = EasyID3)               
            except mutagen.id3.error:
                pass
            ismp3 = True
        elif mfilepath.endswith('ogg') or mfilepath.endswith('OGG'):
            meta = OggVorbis(mfilepath)
            bitrate = meta.info.bitrate
            duration = toString(int(meta.info.length + 0.5))
            ismp3 = False
        
    return meta, ismp3, bitrate, duration
#-------------------------------------------------------------------------------------------

def searchFile(filename):
    """Given a search path, find file
    """
    filefound = False
    filepath = ''
    searchpaths = os.path.expandvars('$PATH')
    paths = searchpaths.split(os.pathsep)
    for path in paths:
        filepath = os.path.join(path, filename)
        if os.path.exists(filepath):
            filefound = True
            break
    return filefound, filepath
#-------------------------------------------------------------------------------------------

def loadIcon(iconname):
    """ small helper function to load icons """
    try:
        bitmap = wx.Bitmap('%s%s%s' % (ICONPATH, os.sep, iconname))
    except:
        bitmap = wx.NullBitmap
    return bitmap
#-------------------------------------------------------------------------------------------

def format3P(vs):                       # Werte wie '12765' in '12.765' formatieren
    vs = str(vs)
    dotpos = len(vs)

    while dotpos > 3:                   # alle 3 Stellen ein '.'
        dotpos -= 3
        vs = '%s.%s' % (vs[0:dotpos], vs[dotpos:])
    return vs
#-------------------------------------------------------------------------------------------

def comboBoxHistory(ctrl, config, section, key):
    txtitem = ctrl.GetValue()
    if not txtitem:
        try:
            if len(config[section][key]) > 1:
                del config[section][key][0]
            txtitem = config[section][key][0]
            ctrl.Clear()
            for item in config[section][key]:
                ctrl.Append(item)
            ctrl.SetValue(txtitem)
        except:
            pass
        
    if txtitem != config[section][key][0]:
        if txtitem in config[section][key]:
            config[section][key].remove(txtitem)
        config[section][key].insert(0, txtitem)
        if len(config[section][key]) == 9:
            del config[section][key][8]
        ctrl.Clear()
        for item in config[section][key]:
            ctrl.Append(item)
        ctrl.SetValue(txtitem)
#-------------------------------------------------------------------------------------------

def findTitleExt(config, titleid):
    path = os.path.join(config['Misc']['musicbase'], titleid[:2], titleid[:4], titleid) + '.'
    found = False
    for ext in config['Misc']['mfileexts']:
        if os.path.exists(path + ext):
            found = True
            break
        ext = ext.upper()
        if os.path.exists(path + ext):
            found = True
            break        
    if found: 
        return True, path + ext
    else:
        return False, path + '???'
#-------------------------------------------------------------------------------------------

def toSeconds(timestr):
    try:
        t = timestr.split(':')
        return int(t[0]) * 3600 + int(t[1]) * 60 + int(t[2])
    except:
        return 0
#-------------------------------------------------------------------------------------------

def toString(timesum):
    h = timesum / 3600
    timesum %= 3600
    m = timesum / 60
    s = timesum % 60
    return '%02i:%02i:%02i' % (h, m, s)
#-------------------------------------------------------------------------------------------

def titleCase(s):
    return re.sub(r"[A-Za-z]+('[A-Za-z]+)?",
                  lambda mo: mo.group(0)[0].upper() + mo.group(0)[1:].lower(), s)
#-------------------------------------------------------------------------------------------

def initQueryExample():
    return {'QuickQuery':  {'name':     'QuickQuery',
                            'cbfuzzy':  'True',
                            'cbtitle':  'False',
                            'cbinter':  'False',
                            'columns':  ['Id_Title', 'Title', 'Interpreter', 'Duration', 'Date', 'Code', 'Composer'],
                            'colwidth': [70, 240, 240, 60, 60, 60, 60],
                            'fuzzyhist':[''],
                            'titlehist':[''],
                            'interhist':[''],   
                            'radiobox': 0,
                           },              
             'AlbumTitles':{'name':     'AlbumTitles',
                            'sql':      'SELECT Id_Title, Title, Interpreter, Duration, \
Date, Code, Composer FROM muma_title WHERE Id_Title LIKE "%" ORDER BY Title',
                            'columns':  ['Id_Title', 'Title', 'Interpreter', 'Duration', 'Date', 'Code', 'Composer'],
                            'orderby':  ['Title'],
                            'colwidth': [50, 200, 200, 70, 80, 70, 200],
                           },
            'All Titles': {'name':      'All Titles',
                           'sql':       'SELECT Id_Title, Title, Interpreter, Duration, \
Date, Code, Composer FROM muma_title WHERE Id_Title LIKE "%" AND Title LIKE "%" AND Interpreter LIKE "%"\
AND Duration LIKE "%" AND Date LIKE "%" AND Code LIKE "%" AND Composer LIKE "%" ORDER BY Title',
                           'columns':   ['Id_Title', 'Title', 'Interpreter', 'Duration', 'Date', 'Code', 'Composer'],
                           'orderby':   ['Title'],
                           'colwidth':  [50, 200, 200, 70, 80, 70, 200],
                           'id_title':  '%',
                           'title':     '%',
                           'interpreter':'%',
                           'duration':   '%',
                           'date':     '%',
                           'code':      '%',
                           'composer': '%',
                           'distinct':  'False',
                           'join':      'False',
                           'hitpar':    'False',
                           'pdate':     'False',
                           'freeformat':'False',
                           'freewhere': 'Id_Title LIKE "%" AND Title LIKE "%" AND Interpreter \
LIKE "%" AND Duration LIKE "%" AND Date LIKE "%" AND Code LIKE "%" AND Composer LIKE "%"' ,
                           'paramhist':  [''],
                           },
             'All CDs':    {'name':     'All CDs',
                            'sql':      'SELECT Id_Title, Title, Interpreter, Duration, Date, \
Code FROM muma_title WHERE Id_Title LIKE "%00" AND Title LIKE "%" AND Interpreter LIKE "%" AND \
Duration LIKE "%" AND Date LIKE "%" AND Code LIKE "%" AND Composer LIKE "%" ORDER BY Id_Title',
                            'columns':  ['Id_Title', 'Title', 'Interpreter', 'Duration', 'Date', 'Code'],
                            'colwidth': [65, 257, 199, 73, 87, 58],
                            'orderby':  ['Id_Title'],
                            'code':     '%',
                            'title':    '%',
                            'duration': '%',
                            'date':     '%',
                            'interpreter':'%',
                            'composer': '%',
                            'paramhist':[''],
                            'join':     'False',
                            'hitpar':   'False',
                            'distinct': 'False',
                            'freewhere':'Id_Title LIKE "%" AND Title LIKE "%" AND Interpreter \
LIKE "%" AND Duration LIKE "%" AND Date LIKE "%" AND Code LIKE "%" AND Composer LIKE "%"',
                            'id_title': '%00',
                            'freeformat':'False',
                           },
             'AddRecord':  {'name':     'AddRecord',
                            'sql':      "SELECT Id_Title, Title, Interpreter, Duration, Date, Code, \
Composer FROM muma_title WHERE Id_Title LIKE '%volume_id%' ORDER BY Id_Title",
                            'columns':  ['Id_Title', 'Title', 'Interpreter', 'Duration', 'Date', 'Code', 'Composer'],
                            'colwidth': [66, 222, 204, 70, 80, 59, 200],
                            'orderby':  ['Id_Title'],
                            'code':     '%',
                            'title':    '%',
                            'duration': '%',
                            'date':     '%',
                            'interpreter':'%',
                            'composer':'%',
                            'paramhist':[''],
                            'join':     'False',
                            'hitpar':   'False',
                            'distinct': 'False',
                            'freewhere':"Id_Title LIKE '%%param%%'",
                            'id_title': '%00',
                            'freeformat':'True',                            
                           },
             'Volume':     {'name':     'Volume',
                            'sql':      "SELECT Id_Title, Title, Interpreter, Duration, Date, Code, \
Composer FROM muma_title WHERE Id_Title LIKE '%%param%%' ORDER BY Id_Title",
                            'columns':  ['Id_Title', 'Title', 'Interpreter', 'Duration', 'Date', 'Code', 'Composer'],
                            'colwidth': [66, 222, 204, 70, 80, 59, 200],
                            'orderby':  ['Id_Title'],
                            'code':     '%',
                            'title':    '%',
                            'duration':     '%',
                            'date':    '%',
                            'interpreter':'%',
                            'composer':'%',
                            'paramhist':[''],
                            'join':     'False',
                            'hitpar':   'False',
                            'distinct': 'False',
                            'freewhere':"Id_Title LIKE '%%param%%'",
                            'id_title': '%00',
                            'freeformat':'True',                            
                           },
             'Charts-date':{'name':     'Charts-date',
                            'sql':      "SELECT Id_Title, Title, Interpreter, Code, Place \
FROM muma_title, muma_place WHERE Pdate BETWEEN date('%param%', '-3 days')AND \
date('%param%', '+3 days') AND Id_Title = Title_Id ORDER BY Place",
                            'columns':  ['Id_Title', 'Title', 'Interpreter', 'Code', 'Place'],
                            'colwidth': [64, 227, 206, 70, 43],
                            'orderby':  ['Place'],
                            'code':     '%',
                            'title':    '%',
                            'duration': '%',
                            'date':     '%',
                            'interpreter':'%',
                            'composer':'%',
                            'paramhist':[''],
                            'join':     'True',
                            'hitpar':   'True',
                            'distinct': 'False',
                            'freewhere':"Pdate LIKE '%param%' AND Title_Id = Id_Title",
                            'id_title': '%',
                            'freeformat':'True',                            
                           },
             'Title':      {'name':     'Title',
                            'sql':      'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE \
Title LIKE "%%param%%" ORDER BY Id_Title',
                            'columns':  ['Id_Title', 'Title', 'Interpreter'],
                            'colwidth': [66, 222, 204],
                            'orderby':  ['Id_Title'],
                            'code':     '%',
                            'title':    '%',
                            'duration': '%',
                            'date':     '%',
                            'interpreter':'%',
                            'composer':'%',
                            'paramhist':[''],
                            'join':     'False',
                            'hitpar':   'False',
                            'distinct': 'False',
                            'freewhere':"Title LIKE '%%param%%'",
                            'id_title': '%',
                            'freeformat':'True',                            
                           },
             'Interpreter':{'name':     'Interpreter',
                            'sql':      "SELECT Id_Title, Title, Interpreter, Duration, Date, Composer \
FROM muma_title WHERE Interpreter LIKE '%%param%%' ORDER BY Title",
                            'columns':  ['Id_Title', 'Title', 'Interpreter', 'Duration', 'Date', 'Composer'],
                            'colwidth': [66, 222, 204, 62, 76, 189],
                            'orderby':  ['Title'],
                            'code':     '%',
                            'title':    '%',
                            'duration': '%',
                            'date':    '%',
                            'interpreter':'%',
                            'composer':'%',
                            'paramhist':[''],
                            'join':     'False',
                            'hitpar':   'False',
                            'distinct': 'False',
                            'freewhere':"Interpreter LIKE '%%param%%'",
                            'id_title': '%',
                            'freeformat':'True',                            
                           },                
           }
#-------------------------------------------------------------------------------------------
        
        